AttachSpec("Algaem.spec");

SetVerbose("JacHypSelmer",2);
SetVerbose("EtaleAlg",1);
SetVerbose("EllSelmer",1);
SetVerbose("ClassGroup",2);
SetECLevel(3);

//A very nice example to show that it is important to try all descents available
//to get a sharp bound on your rank.

d:=&*[u:u in [1..20]|IsPrime(u)];
E:=PolToEllipticCurve(d*(x+1)*(x+5)*(x+3));

//RankBounds makes an unfortunate choice
RankBounds(E);

Q:=NumberField(x-1:DoLinearExtension);
EQ:=BaseChange(E,Q);
phi1:=TwoIsogeny(EQ![-d,0]);
phi2:=TwoIsogeny(EQ![-3*d,0]);
phi3:=TwoIsogeny(EQ![-5*d,0]);

//Note that the other 2-isogenies give a better bound
RankBound(phi1);
RankBound(phi2);
RankBound(phi3);

//But this is still not the best bound we can get:
mul2:=MultiplicationByMMap(EQ,2);
RankBound(mul2);

//We could have got this result simply by calling
RankBound(E:TwoDescent);

//Don't believe it? Use Stoll's program (that is provided David fixes his stuff)
TwoSelmerGroupData(Jacobian(HyperellipticCurve(E)));

//This one is a classic from a paper of Kramer
F:=x^3+977*x^2+976*x;
E:=PolToEllipticCurve(F);
RankBound(E);

d:=109;
Ed:=PolToEllipticCurve(d*F);
RankBound(Ed);

K:=NumberField(x^2-d);
EK:=BaseChange(E,K);
RankBound(EK);

//we see that the rank bound of E and Ed do not add up to the bound on EK
//There must be sha somewhere. In fact it is on Ed:
RankBounds(E);

//A nice example that the number field method can be much faster than
//Cremona's homspaces method
E:=PolToEllipticCurve(x^3+3472875);
time RankBound(E);
time RankBounds(E);

//This is a really challenging example from my thesis. For finding the
//local 2-selmer group at the prime above 2, you have to be really careful
//not to lose yourself in endless computations.
//The long wait at 5 is as of yet unexplained. I think it's the root finder.
//this should improve with a better one.
K<alpha>:=NumberField(x^5-2);
E:=PolToEllipticCurve(x^3-5*x^2+5*x);
EK:=BaseChange(E,K);
phi:=TwoIsogeny(EK![0,0]);
RankBound(phi);

//And finally one example of mutually visualised sha[2]:
F:=x^3-22*x^2+21*x+1;
E:=PolToEllipticCurve(F);
RankBound(E);

d:=2;
Ed:=PolToEllipticCurve(d*F);
RankBound(Ed);

K:=NumberField(x^2-d);
EK:=BaseChange(E,K);
phi:=MultiplicationByMMap(EK,2);

//Incidentally, we illustrate how to use external class group information

//define the field(s) you're going to need
L:=AbsoluteField(NumberField([F,x^2-d]));
//compute your class group (under GRH here)
ClassGroup(L:Proof:="GRH");
//Be sure to call IsogenyMu with a "Fields"parameter before you do anything else
//(we don't really care about the result here)
_:=IsogenyMu(phi:Fields:={L});

//and compute the desired rank bound: No class groups!
RankBound(phi);

//Incidentally, note that these curves share a (Z/2Z)^4 in their sha somewhere
//(at least). If fact, it's distributed:
//(the upper bound of 3 is a mistake here. Geoff knows)
RankBounds(E);
RankBounds(Ed);

//To illustrate the new point-searching capabilities of the package, consider
//the following example. (it's a curve we've used before).

K<alpha>:=NumberField(x^5-2);
E:=PolToEllipticCurve(x^3-5*x^2+5*x);
EK:=BaseChange(E,K);

//First, let's see how much torsion there is on this curve
TorsionBound(EK,20);

//this is only a bound! but often it's sharp. A routine based on division
//polynomials really determines the torsion:
MyTorsionSubgroup(EK);

//let's do a full 2-descent on this curve
phi:=MultiplicationByMMap(EK,2);

//the unique prime above 2
p2:=alpha*IntegerRing(K);

//the standard choice for the parameters makes it unlikely that the routine will
//succeed the first time locally at 2. We first precompute at 2 using a 
//a little more slack.

LocalSelmerGroup(phi,p2:Trials:=6000);

//this info is stored in phi, so now we can just call RankBound.

RankBound(phi);

//As we see, we expect rank 2 (as before). Let's see if we can find points
//on the curve.

L:=MyRationalPoints(EK,10);
#L;

//Whoah! plenty of them. Would that give us (pseudo)generators?
//Notice that this routine, which calls an external C-program, silently did an
//awful lot of work. It checked 10*21^5 combinations for possible X-coordinates
//(i.e. [-10..10] for all basis vectors of an LLL-reduced basis of the ring of
//integers of K times [1..10] for the denominator.

PseudoMordellWeilGroup(phi,10);

//in fact, we could have used the previously computed information in this
//routine too, thus avoiding a painful search for points.

PseudoMordellWeilGroup(phi,0:Xcoors:=[p[1]/p[3]:p in L]);

//the group returned is not necessarily the entire Mordell-Weil group. It's sure
//to be of finite, odd index in the Mordell-Weilgroup, though. Also, it does try
//to pick sensible candidates from the list of candidates (i.e. small in a very
//naive sence)

//Now for something completely different. Root finding for polynomials over
//local fields.
//
//The standard routine is CONSIDERABLY slower than the custom routine
//(which is not quite fool proof yet)
Kp:=LocalField(389,x^2-389,50);
KpY<Y>:=PolynomialRing(Kp);
time Roots((Y^5+4*Y^4+Y^2-3)^3*(Y+1));
time MyRoots((Y^5+4*Y^4+Y^2-3)^3*(Y+1));
