/******************************************************************************
 *
 * newmisc.m
 *
 * date:   26/6/2001
 * author: Nils Bruin
 *
 * Some general features that were missing in MAGMA proper.
 *
 ******************************************************************************/

/**
 ** Random -> FldNumElt
 **
 ** Generate a random element of a number field, uniformly distributed in a box
 ** [-Bound..Bound] wrt the prevailing basis of the number field.
 ** If Denominator is specified too, then the element is divided by a uniformly
 ** distributed integer in [1..Denominator] before returned.
 **/

intrinsic Random(K::FldNum: Bound:=false, Denominator:=1)->FldNumElt
  {Generate a random element of K. The parameter Bound must be given and
  specifies that the coefficients wrt the defining basis of K are chosen
  randomly in [-Bound..Bound]. A random denominator from [1..Denominator] is
  also chosen. Denominator defaults to 1}
  require Bound cmpne false : "Need to specify Bound";
  return K![Random([-Bound..Bound]):i in
               [1..Degree(K)]]/Random([1..Denominator]);
end intrinsic;

/**
 ** OrderedGenerators
 **
 ** Returns an ordered list of generators rather than a set of them.
 ** Indispensible if you want to loop through them in order (for instance, if
 ** you want to define a homomorphism)
 **/
 
intrinsic OrderedGenerators(L::.)->.
  {patch to get an *ordered* list of generators}
  return [L.i:i in [1..Ngens(L)]];
end intrinsic;
