function FormatFilterInput(F,origin,Bound,Primes)
  T:=Cputime();
  O:=BaseRing(F);
  error if not(Type(O) eq RngOrd and IsAbsoluteOrder(O)),
    "Polynomial must be over an absolute order";
  S:="";
  S cat:=Sprintf("%o\n",Degree(O));
  S cat:=Sprintf("%o\n",Bound+1);
  S cat:=Sprintf("%o\n",#Primes);
  for p in Primes do
    error if not(InertiaDegree(p) eq 1),
      "can only handle primes of inertia degree 1";
    Obar,red:=ResidueClassField(p);
    Fbar:=Polynomial(Obar,[red(u):u in Eltseq(F)]);
    S cat:=Sprintf("%o\n",Minimum(p));
    for i:=1 to Degree(O) do
      S cat:=Sprintf("%o ",red(O.i));
    end for;
    S cat:=Sprintf("\n%o\n",red(origin));
    S cat:=&cat[
       IsSquare(Evaluate(Fbar,red(O!x))) select
         "1"
       else
         "0": x in [0..Minimum(p)-1]] cat "\n";
  end for;
  print Cputime()-T;	 
  return S;
end function;

intrinsic NonInertPrimes(O::RngOrd,M::RngOrdIdl,N::RngIntElt)->SeqEnum
  {returns the first N primes of inertia degree 1, where M has 0 valuation}
  p:=1;
  R:=[];
  repeat
    p:=NextPrime(p:Proof:=false);
    S:=Support(O*p);
    for P in S do
      if InertiaDegree(P) eq 1 and Valuation(M,P) eq 0 then
        Append(~R,P);
	if #R ge N then
	  break;
	end if;
      end if;
    end for;
  until #R ge N;
  return R;
end intrinsic;

FilterExec:="/home/bruin/package/Algaem/filter";

intrinsic FindIntegerPoints(F::RngUPolElt,Bound::RngIntElt:Primes:=30)->
  SeqEnum
  {Searches for integer points in a box bounded by Bound}
  O:=BaseRing(F);
  K:=NumberField(O);
  dsc:=O!Discriminant(Polynomial(K,Eltseq(F)));
  require Type(O) eq RngOrd and IsAbsoluteOrder(O):
    "Polynomial must be over an absolute order";
  if Type(Primes) eq RngIntElt then
    Primes:=NonInertPrimes(O,2*dsc*O,Primes);
  end if;
  origin:=-Bound*&+[O.i: i in [1.. Degree(O)]];
  T:=Realtime();
  S:=Pipe(FilterExec,
         FormatFilterInput(F,origin,2*Bound+1,Primes));
  print Realtime()-T;
  Result:=[O!StringToIntegerSequence(s)+origin: s in Split(S,"\n")];
  return [x: x in Result | v eq 0 or IsSquare(v) where v is Evaluate(F,x)];
end intrinsic;

intrinsic MyRationalPoints(F::RngUPolElt,Bound::RngIntElt:
  NPrimes:=30,DenominatorBound:=-1)->SetEnum
  {Finds rational points for y^2=F(x) in a box bounded by Bound}
  if DenominatorBound le 0 then
    DenominatorBound:=Bound;
  end if;
  K:=BaseRing(F);
  require Type(K) eq FldNum: "Polynomial should be over a number field";
  O:=LLL(IntegerRing(K));
  f:=Eltseq(F);
  R:={};
  primes:=NonInertPrimes(O,2*Discriminant(F)*O,NPrimes);
  for den:=1 to DenominatorBound do
    G:=Polynomial(O,[f[i]*den^(#f-i) : i in [1..#f]])*
      ((#f mod 2 eq 0) select den else 1);
    R join:={K!(x/den): x in FindIntegerPoints(G,Bound:Primes:=primes)};
  end for;
  return R;
end intrinsic;

intrinsic MyRationalPoints(E::CrvEll,Bound::RngIntElt:
  NPrimes:=30,DenominatorBound:=-1)->SetEnum
  {Finds rational points for y^2=F(x) in a box bounded by Bound}
  F:=UnivariatePolynomial(E);
  L:=MyRationalPoints(F,Bound:
     NPrimes:=NPrimes,DenominatorBound:=DenominatorBound);
  return {E![x,y] where _,y:=IsSquare(Evaluate(F,x)) : x in L | x ne 0} join
    {E![x[1],0]: x in Roots(F)};
end intrinsic;

function VeryNaiveHeight(a)
 return Abs(Norm(a*Denominator(a))*Denominator(a));
end function;

intrinsic PseudoMordellWeilGroup(phi::Map,Bound::RngIntElt:Xcoors:=[])->GrpAb,Map,SeqEnum
  {tries to find an odd index subgroup of the Mordell-Weil group of the codomain
  of phi using the selmer group of phi and search for points}
  case Degree(phi):
  when 2:
    require false: "Not implemented yet";
  when 4:
    require IsogenyIsMultiplicationByTwo(phi): "Not implemented";
    E:=Codomain(phi);
    mu:=IsogenyMu(phi);
    slmgrp,slm:=SelmerGroup(phi);
    torgrp,tormp:=MyTorsionSubgroup(E);
    torimg:=sub<slmgrp|[slm(mu(tormp(u))):u in OrderedGenerators(torgrp)]>;
    if torimg eq slmgrp then
      return torgrp,tormp,[];
    end if;
    U:=[<x,ysqr>: x in Xcoors | ysqr ne 0 where
         ysqr:=Evaluate(UnivariatePolynomial(E),x)];
    U:=[E![pr[1],y]: pr in U | res where res,y is IsSquare(pr[2])]; 
    L:=Setseq(MyRationalPoints(E,Bound)) cat U;
    Sort(~L,func<a,b|VeryNaiveHeight(a[1]/a[3])-VeryNaiveHeight(b[1]/b[3])>);
    genimg:=torimg;
    R:=[];
    for G in L do
      Gimg:=slm(mu(G));
      if Gimg notin genimg then
	Append(~R,G);
	genimg:=sub<slmgrp|genimg,Gimg>;
	if genimg eq slmgrp then
          break;
	end if;
      end if;
    end for;
    if genimg eq slmgrp then
      MWgrp:=AbelianGroup([0: a in R] cat
                   [Order(G): G in OrderedGenerators(torgrp)]);
      gn:=R cat [tormp(u): u in OrderedGenerators(torgrp)];
      MWgrpmap:=hom<MWgrp->E|
	 a:->&+[gn[i]*a[i] where a is Eltseq(a):i in [1..#gn]]>;
      return MWgrp,MWgrpmap,R;
    else
      error "Failed to find sufficient generators to meet 2-selmer rank bound";
    end if;
  else
    require false: "Not implemented";
  end case;  
end intrinsic;
